@@ -420,7 +420,7 @@ int is_device(char *name)
 	while ((slash = strchr(name, '/'))) {
 		*slash = '!';
 	}
-	snprintf(syspath, sizeof(syspath), "/sys/block/%s", name);
+	snprintf(syspath, sizeof(syspath), "%s/%s", SYSFS_BLOCK, name);
 	
 	return !(access(syspath, F_OK));
 }
